(function (factory) {
    if (typeof define === 'function' && define.amd && define.amd.Zepto) {
        // AMD. Register as anonymous module.
        define(['zepto'], factory);
    } else {
        // Browser globals.
        factory(Zepto);
    }
}(function ($) {
    "use strict";

    //Constants
    var VERSION = "1.6.9",
        LEFT = "left",
        RIGHT = "right",
        UP = "up",
        DOWN = "down",
        IN = "in",
        OUT = "out",

        NONE = "none",
        AUTO = "auto",
        
        SWIPE = "swipe",
        PINCH = "pinch",
        TAP = "tap",
        DOUBLE_TAP = "doubletap",
        LONG_TAP = "longtap",
        HOLD = "hold",
        
        HORIZONTAL = "horizontal",
        VERTICAL = "vertical",

        ALL_FINGERS = "all",
        
        DOUBLE_TAP_THRESHOLD = 10,

        PHASE_START = "start",
        PHASE_MOVE = "move",
        PHASE_END = "end",
        PHASE_CANCEL = "cancel",

        SUPPORTS_TOUCH = 'ontouchstart' in window,
        
        SUPPORTS_POINTER_IE10 = window.navigator.msPointerEnabled && !window.navigator.pointerEnabled,
        
        SUPPORTS_POINTER = window.navigator.pointerEnabled || window.navigator.msPointerEnabled,

        PLUGIN_NS = 'TouchSwipe';

    var defaults = {
        fingers: 1,         
        threshold: 75,  
        cancelThreshold:null,   
        pinchThreshold:20,
        maxTimeThreshold: null, 
        fingerReleaseThreshold:250, 
        longTapThreshold:500,
        doubleTapThreshold:200,
        swipe: null,        
        swipeLeft: null,    
        swipeRight: null,   
        swipeUp: null,      
        swipeDown: null,    
        swipeStatus: null,  
        pinchIn:null,       
        pinchOut:null,      
        pinchStatus:null,   
        click:null, //Deprecated since 1.6.2
        tap:null,
        doubleTap:null,
        longTap:null,       
        hold:null, 
        triggerOnTouchEnd: true, 
        triggerOnTouchLeave:false, 
        allowPageScroll: "auto", 
        fallbackToMouseEvents: true,    
        excludedElements:"label, button, input, select, textarea, a, .noSwipe",
        preventDefaultEvents:true
    };

    $.fn.swipe = function (method) {
        var $this = $(this),
            plugin = $this.data(PLUGIN_NS);

        //Check if we are already instantiated and trying to execute a method   
        if (plugin && typeof method === 'string') {
            if (plugin[method]) {
                return plugin[method].apply(this, Array.prototype.slice.call(arguments, 1));
            } else {
                $.error('Method ' + method + ' does not exist on Zepto.swipe');
            }
        }
        //Else not instantiated and trying to pass init object (or nothing)
        else if (!plugin && (typeof method === 'object' || !method)) {
            return init.apply(this, arguments);
        }

        return $this;
    };
    
    /**
     * The version of the plugin
     * @readonly
     */
    $.fn.swipe.version = VERSION;



    //Expose our defaults so a user could override the plugin defaults
    $.fn.swipe.defaults = defaults;

    /**
    * The phases that a touch event goes through.  The <code>phase</code> is passed to the event handlers. 
    * These properties are read only, attempting to change them will not alter the values passed to the event handlers.
    * @namespace
    * @readonly
    * @property {string} PHASE_START Constant indicating the start phase of the touch event. Value is <code>"start"</code>.
    * @property {string} PHASE_MOVE Constant indicating the move phase of the touch event. Value is <code>"move"</code>.
    * @property {string} PHASE_END Constant indicating the end phase of the touch event. Value is <code>"end"</code>.
    * @property {string} PHASE_CANCEL Constant indicating the cancel phase of the touch event. Value is <code>"cancel"</code>.
    */
    $.fn.swipe.phases = {
        PHASE_START: PHASE_START,
        PHASE_MOVE: PHASE_MOVE,
        PHASE_END: PHASE_END,
        PHASE_CANCEL: PHASE_CANCEL
    };

    /**
    * The direction constants that are passed to the event handlers. 
    * These properties are read only, attempting to change them will not alter the values passed to the event handlers.
    * @namespace
    * @readonly
    * @property {string} LEFT Constant indicating the left direction. Value is <code>"left"</code>.
    * @property {string} RIGHT Constant indicating the right direction. Value is <code>"right"</code>.
    * @property {string} UP Constant indicating the up direction. Value is <code>"up"</code>.
    * @property {string} DOWN Constant indicating the down direction. Value is <code>"cancel"</code>.
    * @property {string} IN Constant indicating the in direction. Value is <code>"in"</code>.
    * @property {string} OUT Constant indicating the out direction. Value is <code>"out"</code>.
    */
    $.fn.swipe.directions = {
        LEFT: LEFT,
        RIGHT: RIGHT,
        UP: UP,
        DOWN: DOWN,
        IN : IN,
        OUT: OUT
    };
    
    /**
    * The page scroll constants that can be used to set the value of <code>allowPageScroll</code> option
    * These properties are read only
    * @namespace
    * @readonly
    * @see $.fn.swipe.defaults#allowPageScroll
    * @property {string} NONE Constant indicating no page scrolling is allowed. Value is <code>"none"</code>.
    * @property {string} HORIZONTAL Constant indicating horizontal page scrolling is allowed. Value is <code>"horizontal"</code>.
    * @property {string} VERTICAL Constant indicating vertical page scrolling is allowed. Value is <code>"vertical"</code>.
    * @property {string} AUTO Constant indicating either horizontal or vertical will be allowed, depending on the swipe handlers registered. Value is <code>"auto"</code>.
    */
    $.fn.swipe.pageScroll = {
        NONE: NONE,
        HORIZONTAL: HORIZONTAL,
        VERTICAL: VERTICAL,
        AUTO: AUTO
    };

    /**
    * Constants representing the number of fingers used in a swipe.  These are used to set both the value of <code>fingers</code> in the 
    * options object, as well as the value of the <code>fingers</code> event property.
    * These properties are read only, attempting to change them will not alter the values passed to the event handlers.
    * @namespace
    * @readonly
    * @see $.fn.swipe.defaults#fingers
    * @property {string} ONE Constant indicating 1 finger is to be detected / was detected. Value is <code>1</code>.
    * @property {string} TWO Constant indicating 2 fingers are to be detected / were detected. Value is <code>1</code>.
    * @property {string} THREE Constant indicating 3 finger are to be detected / were detected. Value is <code>1</code>.
    * @property {string} ALL Constant indicating any combination of finger are to be detected.  Value is <code>"all"</code>.
    */
    $.fn.swipe.fingers = {
        ONE: 1,
        TWO: 2,
        THREE: 3,
        ALL: ALL_FINGERS
    };

    /**
    * Initialise the plugin for each DOM element matched
    * This creates a new instance of the main TouchSwipe class for each DOM element, and then
    * saves a reference to that instance in the elements data property.
    * @internal
    */
    function init(options) {
        //Prep and extend the options
        if (options && (options.allowPageScroll === undefined && (options.swipe !== undefined || options.swipeStatus !== undefined))) {
            options.allowPageScroll = NONE;
        }
        
        //Check for deprecated options
        //Ensure that any old click handlers are assigned to the new tap, unless we have a tap
        if(options.click!==undefined && options.tap===undefined) {
            options.tap = options.click;
        }

        if (!options) {
            options = {};
        }
        
        //pass empty object so we dont modify the defaults
        options = $.extend({}, $.fn.swipe.defaults, options);

        //For each element instantiate the plugin
        return this.each(function () {
            var $this = $(this);

            //Check we havent already initialised the plugin
            var plugin = $this.data(PLUGIN_NS);

            if (!plugin) {
                plugin = new TouchSwipe(this, options);
                $this.data(PLUGIN_NS, plugin);
            }
        });
    }

    /**
    * Main TouchSwipe Plugin Class.
    * Do not use this to construct your TouchSwipe object, use the Zepto plugin method $.fn.swipe(); {@link $.fn.swipe}
    * @private
    * @name TouchSwipe
    * @param {DOMNode} element The HTML DOM object to apply to plugin to
    * @param {Object} options The options to configure the plugin with.  @link {$.fn.swipe.defaults}
    * @see $.fh.swipe.defaults
    * @see $.fh.swipe
    * @class
    */
    function TouchSwipe(element, options) {
        var useTouchEvents = (SUPPORTS_TOUCH || SUPPORTS_POINTER || !options.fallbackToMouseEvents),
            START_EV = useTouchEvents ? (SUPPORTS_POINTER ? (SUPPORTS_POINTER_IE10 ? 'MSPointerDown' : 'pointerdown') : 'touchstart') : 'mousedown',
            MOVE_EV = useTouchEvents ? (SUPPORTS_POINTER ? (SUPPORTS_POINTER_IE10 ? 'MSPointerMove' : 'pointermove') : 'touchmove') : 'mousemove',
            END_EV = useTouchEvents ? (SUPPORTS_POINTER ? (SUPPORTS_POINTER_IE10 ? 'MSPointerUp' : 'pointerup') : 'touchend') : 'mouseup',
            LEAVE_EV = useTouchEvents ? null : 'mouseleave', //we manually detect leave on touch devices, so null event here
            CANCEL_EV = (SUPPORTS_POINTER ? (SUPPORTS_POINTER_IE10 ? 'MSPointerCancel' : 'pointercancel') : 'touchcancel');



        //touch properties
        var distance = 0,
            direction = null,
            duration = 0,
            startTouchesDistance = 0,
            endTouchesDistance = 0,
            pinchZoom = 1,
            pinchDistance = 0,
            pinchDirection = 0,
            maximumsMap=null;

        
        
        //Zepto wrapped element for this instance
        var $element = $(element);
        
        //Current phase of th touch cycle
        var phase = "start";

        // the current number of fingers being used.
        var fingerCount = 0;            

        //track mouse points / delta
        var fingerData=null;

        //track times
        var startTime = 0,
            endTime = 0,
            previousTouchEndTime=0,
            previousTouchFingerCount=0,
            doubleTapStartTime=0;

        //Timeouts
        var singleTapTimeout=null,
            holdTimeout=null;
        
        // Add gestures to all swipable areas if supported
        try {
            $element.bind(START_EV, touchStart);
            $element.bind(CANCEL_EV, touchCancel);
        }
        catch (e) {
            $.error('events not supported ' + START_EV + ',' + CANCEL_EV + ' on Zepto.swipe');
        }

        //
        //Public methods
        //
        
        /**
        * re-enables the swipe plugin with the previous configuration
        * @function
        * @name $.fn.swipe#enable
        * @return {DOMNode} The Dom element that was registered with TouchSwipe 
        * @example $("#element").swipe("enable");
        */
        this.enable = function () {
            $element.bind(START_EV, touchStart);
            $element.bind(CANCEL_EV, touchCancel);
            return $element;
        };

        /**
        * disables the swipe plugin
        * @function
        * @name $.fn.swipe#disable
        * @return {DOMNode} The Dom element that is now registered with TouchSwipe
        * @example $("#element").swipe("disable");
        */
        this.disable = function () {
            removeListeners();
            return $element;
        };

        /**
        * Destroy the swipe plugin completely. To use any swipe methods, you must re initialise the plugin.
        * @function
        * @name $.fn.swipe#destroy
        
        * @example $("#element").swipe("destroy");
        */
        this.destroy = function () {
            removeListeners();
            $element.data(PLUGIN_NS, null);
            $element = null;
        };


        /**
         * Allows run time updating of the swipe configuration options.
         * @function
         * @name $.fn.swipe#option
         * @param {String} property The option property to get or set
         * @param {Object} [value] The value to set the property to
         * @return {Object} If only a property name is passed, then that property value is returned.
         * @example $("#element").swipe("option", "threshold"); // return the threshold
         * @example $("#element").swipe("option", "threshold", 100); // set the threshold after init
         * @see $.fn.swipe.defaults
         *
         */
        this.option = function (property, value) {
            if(options[property]!==undefined) {
                if(value===undefined) {
                    return options[property];
                } else {
                    options[property] = value;
                }
            } else {
                $.error('Option ' + property + ' does not exist on Zepto.swipe.options');
            }

            return null;
        }

        //
        // Private methods
        //
        
        //
        // EVENTS
        //
        /**
        * Event handler for a touch start event.
        * Stops the default click event from triggering and stores where we touched
        * @inner
        * @param {object} zpEvent The normalised Zepto event object.
        */
        function touchStart(zpEvent) {
            //If we already in a touch event (a finger already in use) then ignore subsequent ones..
            if( getTouchInProgress() )
                return;
            
            //Check if this element matches any in the excluded elements selectors,  or its parent is excluded, if so, DON'T swipe
            if( $(zpEvent.target).closest( options.excludedElements, $element ).length>0 ) 
                return;
                
            //As we use Zepto bind for events, we need to target the original event object
            //If these events are being programmatically triggered, we don't have an original event object, so use the Jq one.
            var event = zpEvent.originalEvent ? zpEvent.originalEvent : zpEvent;
            
            var ret,
                touches = event.touches,
                evt = touches ? touches[0] : event;

            phase = PHASE_START;

            //If we support touches, get the finger count
            if (touches) {
                // get the total number of fingers touching the screen
                fingerCount = touches.length;
            }
            //Else this is the desktop, so stop the browser from dragging content
            else {
                zpEvent.preventDefault(); //call this on jq event so we are cross browser
            }

            //clear vars..
            distance = 0;
            direction = null;
            pinchDirection=null;
            duration = 0;
            startTouchesDistance=0;
            endTouchesDistance=0;
            pinchZoom = 1;
            pinchDistance = 0;
            fingerData=createAllFingerData();
            maximumsMap=createMaximumsData();
            cancelMultiFingerRelease();

            
            // check the number of fingers is what we are looking for, or we are capturing pinches
            if (!touches || (fingerCount === options.fingers || options.fingers === ALL_FINGERS) || hasPinches()) {
                // get the coordinates of the touch
                createFingerData( 0, evt );
                startTime = getTimeStamp();
                
                if(fingerCount==2) {
                    //Keep track of the initial pinch distance, so we can calculate the diff later
                    //Store second finger data as start
                    createFingerData( 1, touches[1] );
                    startTouchesDistance = endTouchesDistance = calculateTouchesDistance(fingerData[0].start, fingerData[1].start);
                }
                
                if (options.swipeStatus || options.pinchStatus) {
                    ret = triggerHandler(event, phase);
                }
            }
            else {
                //A touch with more or less than the fingers we are looking for, so cancel
                ret = false; 
            }

            //If we have a return value from the users handler, then return and cancel
            if (ret === false) {
                phase = PHASE_CANCEL;
                triggerHandler(event, phase);
                return ret;
            }
            else {
                if (options.hold) {
                    holdTimeout = setTimeout($.proxy(function() {
                        //Trigger the event
                        $element.trigger('hold', [event.target]);
                        //Fire the callback
                        if(options.hold) {
                            ret = options.hold.call($element, event, event.target);
                        }
                    }, this), options.longTapThreshold );
                }

                setTouchInProgress(true);
            }

            return null;
        };
        
        
        
        /**
        * Event handler for a touch move event. 
        * If we change fingers during move, then cancel the event
        * @inner
        * @param {object} zpEvent The normalised Zepto event object.
        */
        function touchMove(zpEvent) {
            
            //As we use Zepto bind for events, we need to target the original event object
            //If these events are being programmatically triggered, we don't have an original event object, so use the Jq one.
            var event = zpEvent.originalEvent ? zpEvent.originalEvent : zpEvent;
            
            //If we are ending, cancelling, or within the threshold of 2 fingers being released, don't track anything..
            if (phase === PHASE_END || phase === PHASE_CANCEL || inMultiFingerRelease())
                return;

            var ret,
                touches = event.touches,
                evt = touches ? touches[0] : event;
            

            //Update the  finger data 
            var currentFinger = updateFingerData(evt);
            endTime = getTimeStamp();
            
            if (touches) {
                fingerCount = touches.length;
            }

            if (options.hold)
                clearTimeout(holdTimeout);

            phase = PHASE_MOVE;

            //If we have 2 fingers get Touches distance as well
            if(fingerCount==2) {
                
                //Keep track of the initial pinch distance, so we can calculate the diff later
                //We do this here as well as the start event, in case they start with 1 finger, and the press 2 fingers
                if(startTouchesDistance==0) {
                    //Create second finger if this is the first time...
                    createFingerData( 1, touches[1] );
                    
                    startTouchesDistance = endTouchesDistance = calculateTouchesDistance(fingerData[0].start, fingerData[1].start);
                } else {
                    //Else just update the second finger
                    updateFingerData(touches[1]);
                
                    endTouchesDistance = calculateTouchesDistance(fingerData[0].end, fingerData[1].end);
                    pinchDirection = calculatePinchDirection(fingerData[0].end, fingerData[1].end);
                }
                
                
                pinchZoom = calculatePinchZoom(startTouchesDistance, endTouchesDistance);
                pinchDistance = Math.abs(startTouchesDistance - endTouchesDistance);
            }
            
            
            

            if ( (fingerCount === options.fingers || options.fingers === ALL_FINGERS) || !touches || hasPinches() ) {
                
                direction = calculateDirection(currentFinger.start, currentFinger.end);
                
                //Check if we need to prevent default event (page scroll / pinch zoom) or not
                validateDefaultEvent(zpEvent, direction);

                //Distance and duration are all off the main finger
                distance = calculateDistance(currentFinger.start, currentFinger.end);
                duration = calculateDuration();

                //Cache the maximum distance we made in this direction
                setMaxDistance(direction, distance);


                if (options.swipeStatus || options.pinchStatus) {
                    ret = triggerHandler(event, phase);
                }
                
                
                //If we trigger end events when threshold are met, or trigger events when touch leaves element
                if(!options.triggerOnTouchEnd || options.triggerOnTouchLeave) {
                    
                    var inBounds = true;
                    
                    //If checking if we leave the element, run the bounds check (we can use touchleave as its not supported on webkit)
                    if(options.triggerOnTouchLeave) {
                        var bounds = getbounds( this );
                        inBounds = isInBounds( currentFinger.end, bounds );
                    }
                    
                    //Trigger end handles as we swipe if thresholds met or if we have left the element if the user has asked to check these..
                    if(!options.triggerOnTouchEnd && inBounds) {
                        phase = getNextPhase( PHASE_MOVE );
                    } 
                    //We end if out of bounds here, so set current phase to END, and check if its modified 
                    else if(options.triggerOnTouchLeave && !inBounds ) {
                        phase = getNextPhase( PHASE_END );
                    }
                        
                    if(phase==PHASE_CANCEL || phase==PHASE_END) {
                        triggerHandler(event, phase);
                    }               
                }
            }
            else {
                phase = PHASE_CANCEL;
                triggerHandler(event, phase);
            }

            if (ret === false) {
                phase = PHASE_CANCEL;
                triggerHandler(event, phase);
            }
        }



        /**
        * Event handler for a touch end event. 
        * Calculate the direction and trigger events
        * @inner
        * @param {object} zpEvent The normalised Zepto event object.
        */
        function touchEnd(zpEvent) {
            //As we use Zepto bind for events, we need to target the original event object
            //If these events are being programmatically triggered, we don't have an original event object, so use the zp one.
            var event = zpEvent.originalEvent ? zpEvent.originalEvent : zpEvent,
                touches = event.touches;

            //If we are still in a touch with another finger return
            //This allows us to wait a fraction and see if the other finger comes up, if it does within the threshold, then we treat it as a multi release, not a single release.
            if (touches) {
                if(touches.length) {
                    startMultiFingerRelease();
                    return true;
                }
            }
            
            //If a previous finger has been released, check how long ago, if within the threshold, then assume it was a multifinger release.
            //This is used to allow 2 fingers to release fractionally after each other, whilst maintainig the event as containg 2 fingers, not 1
            if(inMultiFingerRelease()) {    
                fingerCount=previousTouchFingerCount;
            }   
        
            //Set end of swipe
            endTime = getTimeStamp();
            
            //Get duration incase move was never fired
            duration = calculateDuration();
            
            //If we trigger handlers at end of swipe OR, we trigger during, but they didnt trigger and we are still in the move phase
            if(didSwipeBackToCancel() || !validateSwipeDistance()) {
                phase = PHASE_CANCEL;
                triggerHandler(event, phase);
            } else if (options.triggerOnTouchEnd || (options.triggerOnTouchEnd == false && phase === PHASE_MOVE)) {
                //call this on jq event so we are cross browser 
                zpEvent.preventDefault(); 
                phase = PHASE_END;
                triggerHandler(event, phase);
            }
            //Special cases - A tap should always fire on touch end regardless,
            //So here we manually trigger the tap end handler by itself
            //We dont run trigger handler as it will re-trigger events that may have fired already
            else if (!options.triggerOnTouchEnd && hasTap()) {
                //Trigger the pinch events...
                phase = PHASE_END;
                triggerHandlerForGesture(event, phase, TAP);
            }
            else if (phase === PHASE_MOVE) {
                phase = PHASE_CANCEL;
                triggerHandler(event, phase);
            }

            setTouchInProgress(false);

            return null;
        }



        /**
        * Event handler for a touch cancel event. 
        * Clears current vars
        * @inner
        */
        function touchCancel() {
            // reset the variables back to default values
            fingerCount = 0;
            endTime = 0;
            startTime = 0;
            startTouchesDistance=0;
            endTouchesDistance=0;
            pinchZoom=1;
            
            //If we were in progress of tracking a possible multi touch end, then re set it.
            cancelMultiFingerRelease();
            
            setTouchInProgress(false);
        }
        
        
        /**
        * Event handler for a touch leave event. 
        * This is only triggered on desktops, in touch we work this out manually
        * as the touchleave event is not supported in webkit
        * @inner
        */
        function touchLeave(zpEvent) {
            var event = zpEvent.originalEvent ? zpEvent.originalEvent : zpEvent;
            
            //If we have the trigger on leave property set....
            if(options.triggerOnTouchLeave) {
                phase = getNextPhase( PHASE_END );
                triggerHandler(event, phase);
            }
        }
        
        /**
        * Removes all listeners that were associated with the plugin
        * @inner
        */
        function removeListeners() {
            $element.unbind(START_EV, touchStart);
            $element.unbind(CANCEL_EV, touchCancel);
            $element.unbind(MOVE_EV, touchMove);
            $element.unbind(END_EV, touchEnd);
            
            //we only have leave events on desktop, we manually calculate leave on touch as its not supported in webkit
            if(LEAVE_EV) { 
                $element.unbind(LEAVE_EV, touchLeave);
            }
            
            setTouchInProgress(false);
        }

        
        /**
         * Checks if the time and distance thresholds have been met, and if so then the appropriate handlers are fired.
         */
        function getNextPhase(currentPhase) {
            
            var nextPhase = currentPhase;
            
            // Ensure we have valid swipe (under time and over distance  and check if we are out of bound...)
            var validTime = validateSwipeTime();
            var validDistance = validateSwipeDistance();
            var didCancel = didSwipeBackToCancel();
                        
            //If we have exceeded our time, then cancel 
            if(!validTime || didCancel) {
                nextPhase = PHASE_CANCEL;
            }
            //Else if we are moving, and have reached distance then end
            else if (validDistance && currentPhase == PHASE_MOVE && (!options.triggerOnTouchEnd || options.triggerOnTouchLeave) ) {
                nextPhase = PHASE_END;
            } 
            //Else if we have ended by leaving and didn't reach distance, then cancel
            else if (!validDistance && currentPhase==PHASE_END && options.triggerOnTouchLeave) {
                nextPhase = PHASE_CANCEL;
            }
            
            return nextPhase;
        }
        
        
        /**
        * Trigger the relevant event handler
        * The handlers are passed the original event, the element that was swiped, and in the case of the catch all handler, the direction that was swiped, "left", "right", "up", or "down"
        * @param {object} event the original event object
        * @param {string} phase the phase of the swipe (start, end cancel etc) {@link $.fn.swipe.phases}
        * @inner
        */
        function triggerHandler(event, phase) {

            var ret,
                touches = event.touches;
            
            //Swipes and pinches are not mutually exclusive - can happend at same time, so need to trigger 2 events potentially
            if( (didSwipe() || hasSwipes()) || (didPinch() || hasPinches()) ) {
                // SWIPE GESTURES
                if(didSwipe() || hasSwipes()) { //hasSwipes as status needs to fire even if swipe is invalid
                    //Trigger the swipe events...
                    ret = triggerHandlerForGesture(event, phase, SWIPE);
                }   

                // PINCH GESTURES (if the above didn't cancel)
                if((didPinch() || hasPinches()) && ret!==false) {
                    //Trigger the pinch events...
                    ret = triggerHandlerForGesture(event, phase, PINCH);
                }
            } else {
             
                // CLICK / TAP (if the above didn't cancel)
                if(didDoubleTap() && ret!==false) {
                    //Trigger the tap events...
                    ret = triggerHandlerForGesture(event, phase, DOUBLE_TAP);
                }
                
                // CLICK / TAP (if the above didn't cancel)
                else if(didLongTap() && ret!==false) {
                    //Trigger the tap events...
                    ret = triggerHandlerForGesture(event, phase, LONG_TAP);
                }

                // CLICK / TAP (if the above didn't cancel)
                else if(didTap() && ret!==false) {
                    //Trigger the tap event..
                    ret = triggerHandlerForGesture(event, phase, TAP);
                }
            }
            
            
            
            // If we are cancelling the gesture, then manually trigger the reset handler
            if (phase === PHASE_CANCEL) {
                touchCancel(event);
            }
            
            // If we are ending the gesture, then manually trigger the reset handler IF all fingers are off
            if(phase === PHASE_END) {
                //If we support touch, then check that all fingers are off before we cancel
                if (touches) {
                    if(!touches.length) {
                        touchCancel(event); 
                    }
                } 
                else {
                    touchCancel(event);
                }
            }
                    
            return ret;
        }
        
        
        
        /**
        * Trigger the relevant event handler
        * The handlers are passed the original event, the element that was swiped, and in the case of the catch all handler, the direction that was swiped, "left", "right", "up", or "down"
        * @param {object} event the original event object
        * @param {string} phase the phase of the swipe (start, end cancel etc) {@link $.fn.swipe.phases}
        * @param {string} gesture the gesture to trigger a handler for : PINCH or SWIPE {@link $.fn.swipe.gestures}
        * @return Boolean False, to indicate that the event should stop propagation, or void.
        * @inner
        */
        function triggerHandlerForGesture(event, phase, gesture) {  
            
            var ret;
            
            //SWIPES....
            if(gesture==SWIPE) {
                //Trigger status every time..
                
                //Trigger the event...
                $element.trigger('swipeStatus', [phase, direction || null, distance || 0, duration || 0, fingerCount, fingerData]);
                
                //Fire the callback
                if (options.swipeStatus) {
                    ret = options.swipeStatus.call($element, event, phase, direction || null, distance || 0, duration || 0, fingerCount, fingerData);
                    //If the status cancels, then dont run the subsequent event handlers..
                    if(ret===false) return false;
                }
                
                
                
                
                if (phase == PHASE_END && validateSwipe()) {
                    //Fire the catch all event
                    $element.trigger('swipe', [direction, distance, duration, fingerCount, fingerData]);
                    
                    //Fire catch all callback
                    if (options.swipe) {
                        ret = options.swipe.call($element, event, direction, distance, duration, fingerCount, fingerData);
                        //If the status cancels, then dont run the subsequent event handlers..
                        if(ret===false) return false;
                    }
                    
                    //trigger direction specific event handlers 
                    switch (direction) {
                        case LEFT:
                            //Trigger the event
                            $element.trigger('swipeLeft', [direction, distance, duration, fingerCount, fingerData]);
                    
                            //Fire the callback
                            if (options.swipeLeft) {
                                ret = options.swipeLeft.call($element, event, direction, distance, duration, fingerCount, fingerData);
                            }
                            break;
    
                        case RIGHT:
                            //Trigger the event
                            $element.trigger('swipeRight', [direction, distance, duration, fingerCount, fingerData]);
                    
                            //Fire the callback
                            if (options.swipeRight) {
                                ret = options.swipeRight.call($element, event, direction, distance, duration, fingerCount, fingerData);
                            }
                            break;
    
                        case UP:
                            //Trigger the event
                            $element.trigger('swipeUp', [direction, distance, duration, fingerCount, fingerData]);
                    
                            //Fire the callback
                            if (options.swipeUp) {
                                ret = options.swipeUp.call($element, event, direction, distance, duration, fingerCount, fingerData);
                            }
                            break;
    
                        case DOWN:
                            //Trigger the event
                            $element.trigger('swipeDown', [direction, distance, duration, fingerCount, fingerData]);
                    
                            //Fire the callback
                            if (options.swipeDown) {
                                ret = options.swipeDown.call($element, event, direction, distance, duration, fingerCount, fingerData);
                            }
                            break;
                    }
                }
            }
            
            
            //PINCHES....
            if(gesture==PINCH) {
                //Trigger the event
                 $element.trigger('pinchStatus', [phase, pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData]);
                    
                //Fire the callback
                if (options.pinchStatus) {
                    ret = options.pinchStatus.call($element, event, phase, pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData);
                    //If the status cancels, then dont run the subsequent event handlers..
                    if(ret===false) return false;
                }
                
                if(phase==PHASE_END && validatePinch()) {
                    
                    switch (pinchDirection) {
                        case IN:
                            //Trigger the event
                            $element.trigger('pinchIn', [pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData]);
                    
                            //Fire the callback
                            if (options.pinchIn) {
                                ret = options.pinchIn.call($element, event, pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData);
                            }
                            break;
                        
                        case OUT:
                            //Trigger the event
                            $element.trigger('pinchOut', [pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData]);
                    
                            //Fire the callback
                            if (options.pinchOut) {
                                ret = options.pinchOut.call($element, event, pinchDirection || null, pinchDistance || 0, duration || 0, fingerCount, pinchZoom, fingerData);
                            }
                            break;  
                    }
                }
            }
            


                
                
            if(gesture==TAP) {
                if(phase === PHASE_CANCEL || phase === PHASE_END) {
                    
                    
                    //Cancel any existing double tap
                    clearTimeout(singleTapTimeout);
                    //Cancel hold timeout
                    clearTimeout(holdTimeout);
                           
                    //If we are also looking for doubelTaps, wait incase this is one...
                    if(hasDoubleTap() && !inDoubleTap()) {
                        //Cache the time of this tap
                        doubleTapStartTime = getTimeStamp();
                       
                        //Now wait for the double tap timeout, and trigger this single tap
                        //if its not cancelled by a double tap
                        singleTapTimeout = setTimeout($.proxy(function() {
                            doubleTapStartTime=null;
                            //Trigger the event
                            $element.trigger('tap', [event.target]);

                        
                            //Fire the callback
                            if(options.tap) {
                                ret = options.tap.call($element, event, event.target);
                            }
                        }, this), options.doubleTapThreshold );
                        
                    } else {
                        doubleTapStartTime=null;
                        
                        //Trigger the event
                        $element.trigger('tap', [event.target]);

                        
                        //Fire the callback
                        if(options.tap) {
                            ret = options.tap.call($element, event, event.target);
                        }
                    }
                }
            }
            
            else if (gesture==DOUBLE_TAP) {
                if(phase === PHASE_CANCEL || phase === PHASE_END) {
                    //Cancel any pending singletap 
                    clearTimeout(singleTapTimeout);
                    doubleTapStartTime=null;
                        
                    //Trigger the event
                    $element.trigger('doubletap', [event.target]);
                
                    //Fire the callback
                    if(options.doubleTap) {
                        ret = options.doubleTap.call($element, event, event.target);
                    }
                }
            }
            
            else if (gesture==LONG_TAP) {
                if(phase === PHASE_CANCEL || phase === PHASE_END) {
                    //Cancel any pending singletap (shouldnt be one)
                    clearTimeout(singleTapTimeout);
                    doubleTapStartTime=null;
                        
                    //Trigger the event
                    $element.trigger('longtap', [event.target]);
                
                    //Fire the callback
                    if(options.longTap) {
                        ret = options.longTap.call($element, event, event.target);
                    }
                }
            }               
                
            return ret;
        }



        
        //
        // GESTURE VALIDATION
        //
        
        /**
        * Checks the user has swipe far enough
        * @return Boolean if <code>threshold</code> has been set, return true if the threshold was met, else false.
        * If no threshold was set, then we return true.
        * @inner
        */
        function validateSwipeDistance() {
            var valid = true;
            //If we made it past the min swipe distance..
            if (options.threshold !== null) {
                valid = distance >= options.threshold;
            }
            
            return valid;
        }
        
        /**
        * Checks the user has swiped back to cancel.
        * @return Boolean if <code>cancelThreshold</code> has been set, return true if the cancelThreshold was met, else false.
        * If no cancelThreshold was set, then we return true.
        * @inner
        */
        function didSwipeBackToCancel() {
            var cancelled = false;
            if(options.cancelThreshold !== null && direction !==null)  {
                cancelled =  (getMaxDistance( direction ) - distance) >= options.cancelThreshold;
            }
            
            return cancelled;
        }

        /**
        * Checks the user has pinched far enough
        * @return Boolean if <code>pinchThreshold</code> has been set, return true if the threshold was met, else false.
        * If no threshold was set, then we return true.
        * @inner
        */
        function validatePinchDistance() {
            if (options.pinchThreshold !== null) {
                return pinchDistance >= options.pinchThreshold;
            }
            return true;
        }

        /**
        * Checks that the time taken to swipe meets the minimum / maximum requirements
        * @return Boolean
        * @inner
        */
        function validateSwipeTime() {
            var result;
            //If no time set, then return true

            if (options.maxTimeThreshold) {
                if (duration >= options.maxTimeThreshold) {
                    result = false;
                } else {
                    result = true;
                }
            }
            else {
                result = true;
            }

            return result;
        }


        /**
        * Checks direction of the swipe and the value allowPageScroll to see if we should allow or prevent the default behaviour from occurring.
        * This will essentially allow page scrolling or not when the user is swiping on a touchSwipe object.
        * @param {object} zpEvent The normalised Zepto representation of the event object.
        * @param {string} direction The direction of the event. See {@link $.fn.swipe.directions}
        * @see $.fn.swipe.directions
        * @inner
        */
        function validateDefaultEvent(zpEvent, direction) {

            if( options.preventDefaultEvents === false ) {
                return;
            }
            
            if (options.allowPageScroll === NONE) {
                zpEvent.preventDefault();
            } else {
                var auto = options.allowPageScroll === AUTO;

                switch (direction) {
                    case LEFT:
                        if ((options.swipeLeft && auto) || (!auto && options.allowPageScroll != HORIZONTAL)) {
                            zpEvent.preventDefault();
                        }
                        break;

                    case RIGHT:
                        if ((options.swipeRight && auto) || (!auto && options.allowPageScroll != HORIZONTAL)) {
                            zpEvent.preventDefault();
                        }
                        break;

                    case UP:
                        if ((options.swipeUp && auto) || (!auto && options.allowPageScroll != VERTICAL)) {
                            zpEvent.preventDefault();
                        }
                        break;

                    case DOWN:
                        if ((options.swipeDown && auto) || (!auto && options.allowPageScroll != VERTICAL)) {
                            zpEvent.preventDefault();
                        }
                        break;
                }
            }

        }


        // PINCHES
        /**
         * Returns true of the current pinch meets the thresholds
         * @return Boolean
         * @inner
        */
        function validatePinch() {
            var hasCorrectFingerCount = validateFingers();
            var hasEndPoint = validateEndPoint();
            var hasCorrectDistance = validatePinchDistance();
            return hasCorrectFingerCount && hasEndPoint && hasCorrectDistance;
            
        }
        
        /**
         * Returns true if any Pinch events have been registered
         * @return Boolean
         * @inner
        */
        function hasPinches() {
            //Enure we dont return 0 or null for false values
            return !!(options.pinchStatus || options.pinchIn || options.pinchOut);
        }
        
        /**
         * Returns true if we are detecting pinches, and have one
         * @return Boolean
         * @inner
         */
        function didPinch() {
            //Enure we dont return 0 or null for false values
            return !!(validatePinch() && hasPinches());
        }




        // SWIPES
        /**
         * Returns true if the current swipe meets the thresholds
         * @return Boolean
         * @inner
        */
        function validateSwipe() {
            //Check validity of swipe
            var hasValidTime = validateSwipeTime();
            var hasValidDistance = validateSwipeDistance(); 
            var hasCorrectFingerCount = validateFingers();
            var hasEndPoint = validateEndPoint();
            var didCancel = didSwipeBackToCancel(); 
            
            // if the user swiped more than the minimum length, perform the appropriate action
            // hasValidDistance is null when no distance is set 
            var valid =  !didCancel && hasEndPoint && hasCorrectFingerCount && hasValidDistance && hasValidTime;
            
            return valid;
        }
        
        /**
         * Returns true if any Swipe events have been registered
         * @return Boolean
         * @inner
        */
        function hasSwipes() {
            //Enure we dont return 0 or null for false values
            return !!(options.swipe || options.swipeStatus || options.swipeLeft || options.swipeRight || options.swipeUp || options.swipeDown);
        }
        
        
        /**
         * Returns true if we are detecting swipes and have one
         * @return Boolean
         * @inner
        */
        function didSwipe() {
            //Enure we dont return 0 or null for false values
            return !!(validateSwipe() && hasSwipes());
        }

        /**
         * Returns true if we have matched the number of fingers we are looking for
         * @return Boolean
         * @inner
        */
        function validateFingers() {
            //The number of fingers we want were matched, or on desktop we ignore
            return ((fingerCount === options.fingers || options.fingers === ALL_FINGERS) || !SUPPORTS_TOUCH);
        }
        
        /**
         * Returns true if we have an end point for the swipe
         * @return Boolean
         * @inner
        */
        function validateEndPoint() {
            //We have an end value for the finger
            return fingerData[0].end.x !== 0;
        }

        // TAP / CLICK
        /**
         * Returns true if a click / tap events have been registered
         * @return Boolean
         * @inner
        */
        function hasTap() {
            //Enure we dont return 0 or null for false values
            return !!(options.tap) ;
        }
        
        /**
         * Returns true if a double tap events have been registered
         * @return Boolean
         * @inner
        */
        function hasDoubleTap() {
            //Enure we dont return 0 or null for false values
            return !!(options.doubleTap) ;
        }
        
        /**
         * Returns true if any long tap events have been registered
         * @return Boolean
         * @inner
        */
        function hasLongTap() {
            //Enure we dont return 0 or null for false values
            return !!(options.longTap) ;
        }
        
        /**
         * Returns true if we could be in the process of a double tap (one tap has occurred, we are listening for double taps, and the threshold hasn't past.
         * @return Boolean
         * @inner
        */
        function validateDoubleTap() {
            if(doubleTapStartTime==null){
                return false;
            }
            var now = getTimeStamp();
            return (hasDoubleTap() && ((now-doubleTapStartTime) <= options.doubleTapThreshold));
        }
        
        /**
         * Returns true if we could be in the process of a double tap (one tap has occurred, we are listening for double taps, and the threshold hasn't past.
         * @return Boolean
         * @inner
        */
        function inDoubleTap() {
            return validateDoubleTap();
        }
        
        
        /**
         * Returns true if we have a valid tap
         * @return Boolean
         * @inner
        */
        function validateTap() {
            return ((fingerCount === 1 || !SUPPORTS_TOUCH) && (isNaN(distance) || distance < options.threshold));
        }
        
        /**
         * Returns true if we have a valid long tap
         * @return Boolean
         * @inner
        */
        function validateLongTap() {
            //slight threshold on moving finger
            return ((duration > options.longTapThreshold) && (distance < DOUBLE_TAP_THRESHOLD)); 
        }
        
        /**
         * Returns true if we are detecting taps and have one
         * @return Boolean
         * @inner
        */
        function didTap() {
            //Enure we dont return 0 or null for false values
            return !!(validateTap() && hasTap());
        }
        
        
        /**
         * Returns true if we are detecting double taps and have one
         * @return Boolean
         * @inner
        */
        function didDoubleTap() {
            //Enure we dont return 0 or null for false values
            return !!(validateDoubleTap() && hasDoubleTap());
        }
        
        /**
         * Returns true if we are detecting long taps and have one
         * @return Boolean
         * @inner
        */
        function didLongTap() {
            //Enure we dont return 0 or null for false values
            return !!(validateLongTap() && hasLongTap());
        }
        
        
        
        
        // MULTI FINGER TOUCH
        /**
         * Starts tracking the time between 2 finger releases, and keeps track of how many fingers we initially had up
         * @inner
        */
        function startMultiFingerRelease() {
            previousTouchEndTime = getTimeStamp();
            previousTouchFingerCount = event.touches.length+1;
        }
        
        /**
         * Cancels the tracking of time between 2 finger releases, and resets counters
         * @inner
        */
        function cancelMultiFingerRelease() {
            previousTouchEndTime = 0;
            previousTouchFingerCount = 0;
        }
        
        /**
         * Checks if we are in the threshold between 2 fingers being released 
         * @return Boolean
         * @inner
        */
        function inMultiFingerRelease() {
            
            var withinThreshold = false;
            
            if(previousTouchEndTime) {  
                var diff = getTimeStamp() - previousTouchEndTime    
                if( diff<=options.fingerReleaseThreshold ) {
                    withinThreshold = true;
                }
            }
            
            return withinThreshold; 
        }
        

        /**
        * gets a data flag to indicate that a touch is in progress
        * @return Boolean
        * @inner
        */
        function getTouchInProgress() {
            //strict equality to ensure only true and false are returned
            return !!($element.data(PLUGIN_NS+'_intouch') === true);
        }
        
        /**
        * Sets a data flag to indicate that a touch is in progress
        * @param {boolean} val The value to set the property to
        * @inner
        */
        function setTouchInProgress(val) {
            
            //Add or remove event listeners depending on touch status
            if(val===true) {
                $element.bind(MOVE_EV, touchMove);
                $element.bind(END_EV, touchEnd);
                
                //we only have leave events on desktop, we manually calcuate leave on touch as its not supported in webkit
                if(LEAVE_EV) { 
                    $element.bind(LEAVE_EV, touchLeave);
                }
            } else {
                $element.unbind(MOVE_EV, touchMove, false);
                $element.unbind(END_EV, touchEnd, false);
            
                //we only have leave events on desktop, we manually calcuate leave on touch as its not supported in webkit
                if(LEAVE_EV) { 
                    $element.unbind(LEAVE_EV, touchLeave, false);
                }
            }
            
        
            //strict equality to ensure only true and false can update the value
            $element.data(PLUGIN_NS+'_intouch', val === true);
        }
        
        
        /**
         * Creates the finger data for the touch/finger in the event object.
         * @param {int} index The index in the array to store the finger data (usually the order the fingers were pressed)
         * @param {object} evt The event object containing finger data
         * @return finger data object
         * @inner
        */
        function createFingerData( index, evt ) {
            var id = evt.identifier!==undefined ? evt.identifier : 0; 
            
            fingerData[index].identifier = id;
            fingerData[index].start.x = fingerData[index].end.x = evt.pageX||evt.clientX;
            fingerData[index].start.y = fingerData[index].end.y = evt.pageY||evt.clientY;
            
            return fingerData[index];
        }
        
        /**
         * Updates the finger data for a particular event object
         * @param {object} evt The event object containing the touch/finger data to upadte
         * @return a finger data object.
         * @inner
        */
        function updateFingerData(evt) {
            
            var id = evt.identifier!==undefined ? evt.identifier : 0; 
            var f = getFingerData( id );
            
            f.end.x = evt.pageX||evt.clientX;
            f.end.y = evt.pageY||evt.clientY;
            
            return f;
        }
        
        /**
         * Returns a finger data object by its event ID.
         * Each touch event has an identifier property, which is used 
         * to track repeat touches
         * @param {int} id The unique id of the finger in the sequence of touch events.
         * @return a finger data object.
         * @inner
        */
        function getFingerData( id ) {
            for(var i=0; i<fingerData.length; i++) {
                if(fingerData[i].identifier == id) {
                    return fingerData[i];   
                }
            }
        }
        
        /**
         * Creats all the finger onjects and returns an array of finger data
         * @return Array of finger objects
         * @inner
        */
        function createAllFingerData() {
            var fingerData=[];
            for (var i=0; i<=5; i++) {
                fingerData.push({
                    start:{ x: 0, y: 0 },
                    end:{ x: 0, y: 0 },
                    identifier:0
                });
            }
            
            return fingerData;
        }
        
        /**
         * Sets the maximum distance swiped in the given direction. 
         * If the new value is lower than the current value, the max value is not changed.
         * @param {string}  direction The direction of the swipe
         * @param {int}  distance The distance of the swipe
         * @inner
        */
        function setMaxDistance(direction, distance) {
            distance = Math.max(distance, getMaxDistance(direction) );
            maximumsMap[direction].distance = distance;
        }
        
        /**
         * gets the maximum distance swiped in the given direction. 
         * @param {string}  direction The direction of the swipe
         * @return int  The distance of the swipe
         * @inner
        */        
        function getMaxDistance(direction) {
            if (maximumsMap[direction]) return maximumsMap[direction].distance;
            return undefined;
        }
        
        /**
         * Creats a map of directions to maximum swiped values.
         * @return Object A dictionary of maximum values, indexed by direction.
         * @inner
        */
        function createMaximumsData() {
            var maxData={};
            maxData[LEFT]=createMaximumVO(LEFT);
            maxData[RIGHT]=createMaximumVO(RIGHT);
            maxData[UP]=createMaximumVO(UP);
            maxData[DOWN]=createMaximumVO(DOWN);
            
            return maxData;
        }
        
        /**
         * Creates a map maximum swiped values for a given swipe direction
         * @param {string} The direction that these values will be associated with
         * @return Object Maximum values
         * @inner
        */
        function createMaximumVO(dir) {
            return { 
                direction:dir, 
                distance:0
            }
        }
        
        
        //
        // MATHS / UTILS
        //

        /**
        * Calculate the duration of the swipe
        * @return int
        * @inner
        */
        function calculateDuration() {
            return endTime - startTime;
        }
        
        /**
        * Calculate the distance between 2 touches (pinch)
        * @param {point} startPoint A point object containing x and y co-ordinates
        * @param {point} endPoint A point object containing x and y co-ordinates
        * @return int;
        * @inner
        */
        function calculateTouchesDistance(startPoint, endPoint) {
            var diffX = Math.abs(startPoint.x - endPoint.x);
            var diffY = Math.abs(startPoint.y - endPoint.y);
                
            return Math.round(Math.sqrt(diffX*diffX+diffY*diffY));
        }
        
        /**
        * Calculate the zoom factor between the start and end distances
        * @param {int} startDistance Distance (between 2 fingers) the user started pinching at
        * @param {int} endDistance Distance (between 2 fingers) the user ended pinching at
        * @return float The zoom value from 0 to 1.
        * @inner
        */
        function calculatePinchZoom(startDistance, endDistance) {
            var percent = (endDistance/startDistance) * 1;
            return percent.toFixed(2);
        }
        
        
        /**
        * Returns the pinch direction, either IN or OUT for the given points
        * @return string Either {@link $.fn.swipe.directions.IN} or {@link $.fn.swipe.directions.OUT}
        * @see $.fn.swipe.directions
        * @inner
        */
        function calculatePinchDirection() {
            if(pinchZoom<1) {
                return OUT;
            }
            else {
                return IN;
            }
        }
        
        
        /**
        * Calculate the length / distance of the swipe
        * @param {point} startPoint A point object containing x and y co-ordinates
        * @param {point} endPoint A point object containing x and y co-ordinates
        * @return int
        * @inner
        */
        function calculateDistance(startPoint, endPoint) {
            return Math.round(Math.sqrt(Math.pow(endPoint.x - startPoint.x, 2) + Math.pow(endPoint.y - startPoint.y, 2)));
        }

        /**
        * Calculate the angle of the swipe
        * @param {point} startPoint A point object containing x and y co-ordinates
        * @param {point} endPoint A point object containing x and y co-ordinates
        * @return int
        * @inner
        */
        function calculateAngle(startPoint, endPoint) {
            var x = startPoint.x - endPoint.x;
            var y = endPoint.y - startPoint.y;
            var r = Math.atan2(y, x); //radians
            var angle = Math.round(r * 180 / Math.PI); //degrees

            //ensure value is positive
            if (angle < 0) {
                angle = 360 - Math.abs(angle);
            }

            return angle;
        }

        /**
        * Calculate the direction of the swipe
        * This will also call calculateAngle to get the latest angle of swipe
        * @param {point} startPoint A point object containing x and y co-ordinates
        * @param {point} endPoint A point object containing x and y co-ordinates
        * @return string Either {@link $.fn.swipe.directions.LEFT} / {@link $.fn.swipe.directions.RIGHT} / {@link $.fn.swipe.directions.DOWN} / {@link $.fn.swipe.directions.UP}
        * @see $.fn.swipe.directions
        * @inner
        */
        function calculateDirection(startPoint, endPoint ) {
            var angle = calculateAngle(startPoint, endPoint);

            if ((angle <= 45) && (angle >= 0)) {
                return LEFT;
            } else if ((angle <= 360) && (angle >= 315)) {
                return LEFT;
            } else if ((angle >= 135) && (angle <= 225)) {
                return RIGHT;
            } else if ((angle > 45) && (angle < 135)) {
                return DOWN;
            } else {
                return UP;
            }
        }
        

        /**
        * Returns a MS time stamp of the current time
        * @return int
        * @inner
        */
        function getTimeStamp() {
            var now = new Date();
            return now.getTime();
        }
        
        
        
        /**
         * Returns a bounds object with left, right, top and bottom properties for the element specified.
         * @param {DomNode} The DOM node to get the bounds for.
         */
        function getbounds( el ) {
            el = $(el);
            var offset = el.offset();
            
            var bounds = {  
                    left:offset.left,
                    right:offset.left+el.outerWidth(),
                    top:offset.top,
                    bottom:offset.top+el.outerHeight()
                    }
            
            return bounds;  
        }
        
        
        /**
         * Checks if the point object is in the bounds object.
         * @param {object} point A point object.
         * @param {int} point.x The x value of the point.
         * @param {int} point.y The x value of the point.
         * @param {object} bounds The bounds object to test
         * @param {int} bounds.left The leftmost value
         * @param {int} bounds.right The righttmost value
         * @param {int} bounds.top The topmost value
        * @param {int} bounds.bottom The bottommost value
         */
        function isInBounds(point, bounds) {
            return (point.x > bounds.left && point.x < bounds.right && point.y > bounds.top && point.y < bounds.bottom);
        };
    
    
    }

}));
